@@ -24,12 +24,12 @@ import random
 
 import numpy as np
 import tables
-import progressbar
 
-from .base import BaseSimulation
+from .detector import HiSPARCSimulation, ErrorlessSimulation
+from ..utils import pbar
 
 
-class GroundParticlesSimulation(BaseSimulation):
+class GroundParticlesSimulation(HiSPARCSimulation):
 
     def __init__(self, corsikafile_path, max_core_distance, *args, **kwargs):
         super(GroundParticlesSimulation, self).__init__(*args, **kwargs)
@@ -39,11 +39,11 @@ class GroundParticlesSimulation(BaseSimulation):
         self.max_core_distance = max_core_distance
 
         for station in self.cluster.stations:
-            station.gps_offset = np.random.normal(0, 16.05)
-            # The actual distribution is not very clear.
-            # We assume it is gaussian for convenience.
-            # Then the stddev is about 16.05
+            station.gps_offset = self.simulate_station_offset()
+            station.detector_offsets = self.simulate_detector_offsets(
+                len(station.detectors))
 
+        # Store updated version of the cluster
         self.coincidence_group._v_attrs.cluster = self.cluster
 
     def __del__(self):
@@ -75,10 +75,6 @@ class GroundParticlesSimulation(BaseSimulation):
                               'energy': event_header.energy,
                               'particle': event_header.particle}
 
-        pbar = progressbar.ProgressBar(widgets=[progressbar.Percentage(),
-                                                progressbar.Bar(),
-                                                progressbar.ETA()])
-
         # DF: This is the fastest implementation I could come up with.  I
         # timed several permutations of numpy / math, and tried a Monte
         # Carlo method in which I pick x and y in a square (fast) and then
@@ -89,20 +85,19 @@ class GroundParticlesSimulation(BaseSimulation):
         for i in pbar(range(self.N)):
             r = sqrt(np.random.uniform(0, R ** 2))
             phi = np.random.uniform(-pi, pi)
-            alpha = np.random.uniform(-pi, pi)
-
             x = r * cos(phi)
             y = r * sin(phi)
 
-            # Add Corsika shower azimuth to generated alpha
-            # and make them fit in (-pi, pi].
-            shower_azimuth = alpha + event_header.azimuth
-            if shower_azimuth > pi:
-                shower_azimuth -= 2 * pi
-            elif shower_azimuth <= -pi:
-                shower_azimuth += 2 * pi
+            # Subtract Corsika shower azimuth from desired shower azimuth
+            # make it fit in (-pi, pi] to get rotation angle of the cluster.
+            shower_azimuth = self.generate_azimuth()
+            alpha = shower_azimuth - event_header.azimuth
+            if alpha > pi:
+                alpha -= 2 * pi
+            elif alpha <= -pi:
+                alpha += 2 * pi
 
-            shower_parameters = {'ext_timestamp': (long(1e9) + i) * long(1e9),
+            shower_parameters = {'ext_timestamp': (int(1e9) + i) * int(1e9),
                                  'core_pos': (x, y),
                                  'azimuth': shower_azimuth}
             shower_parameters.update(corsika_parameters)
@@ -122,7 +117,7 @@ class GroundParticlesSimulation(BaseSimulation):
         xp = x * cos(-alpha) - y * sin(-alpha)
         yp = x * sin(-alpha) + y * cos(-alpha)
 
-        self.cluster.set_xyalpha_coordinates(-xp, -yp, -alpha)
+        self.cluster.set_coordinates(-xp, -yp, 0, -alpha)
 
     def simulate_detector_response(self, detector, shower_parameters):
         """Simulate detector response to a shower.
@@ -131,52 +126,21 @@ class GroundParticlesSimulation(BaseSimulation):
         of leptons in the detector and the arrival time of the first lepton
         passing the detector.
 
-        The detector is approximated by a square with a surface of 0.5
-        square meter which is *not* correctly rotated.  In fact, during
-        the simulation, the rotation of the detector is undefined.  This
-        might be faster than a more thorough implementation.
-
         """
-        detector_boundary = 0.3535534
-        x, y = detector.get_xy_coordinates()
+        particles = self.get_particles_in_detector(detector)
+        n_detected = len(particles)
 
-        # particle ids 2, 3, 5, 6 are electrons and muons, and id 4 is no
-        # longer used (were neutrino's).
-        query = ('(x >= %f) & (x <= %f) & (y >= %f) & (y <= %f)'
-                 ' & (particle_id >= 2) & (particle_id <= 6)' %
-                 (x - detector_boundary, x + detector_boundary,
-                  y - detector_boundary, x + detector_boundary))
-        detected = [row['t'] for row in self.groundparticles.where(query)]
-
-        if detected:
-            n_detected = len(detected)
-            transporttimes = self.simulate_signal_transport_time(n_detected)
-            for i in range(n_detected):
-                detected[i] += transporttimes[i]
-            observables = {'n': n_detected, 't': min(detected)}
+        if n_detected:
+            mips = self.simulate_detector_mips(particles)
+            particles['t'] += self.simulate_signal_transport_time(n_detected)
+            first_signal = particles['t'].min()
+            observables = {'n': mips,
+                           't': self.simulate_adc_sampling(first_signal)}
         else:
             observables = {'n': 0., 't': -999}
 
         return observables
 
-    def simulate_signal_transport_time(self, size):
-        """ Simulate transport times of scintillation light to the PMT
-
-        Generates random transit times within a given distribution and adds it
-        to the times the particles passed the detector.
-
-        """
-        numbers = np.random.random(size)
-        dt = []
-
-        for x in numbers:
-            if  x < 0.39377:
-                dt.append(2.5507 + 2.39885 * x)
-            else:
-                dt.append(1.56764 + 4.89536 * x)
-
-        return dt
-
     def simulate_trigger(self, detector_observables):
         """Simulate a trigger response.
 
@@ -202,10 +166,11 @@ class GroundParticlesSimulation(BaseSimulation):
         if len(arrival_times) > 1:
             trigger_time = sorted(arrival_times)[1]
 
-            timestamp = int(shower_parameters['ext_timestamp'] / 1000000000)
-            nanoseconds = int(trigger_time + self.simulate_gps_uncertainty() +
-                              station.gps_offset)
-            ext_timestamp = shower_parameters['ext_timestamp'] + nanoseconds
+            ext_timestamp = shower_parameters['ext_timestamp']
+            ext_timestamp += int(trigger_time + station.gps_offset +
+                                self.simulate_gps_uncertainty())
+            timestamp = int(ext_timestamp / int(1e9))
+            nanoseconds = int(ext_timestamp % int(1e9))
 
             gps_timestamp = {'ext_timestamp': ext_timestamp,
                              'timestamp': timestamp, 'nanoseconds': nanoseconds}
@@ -213,10 +178,29 @@ class GroundParticlesSimulation(BaseSimulation):
 
         return station_observables
 
-    def simulate_gps_uncertainty(self):
-        """Simulate uncertainty from GPS receiver"""
+    def get_particles_in_detector(self, detector):
+        """Get particles that hit a detector.
 
-        return np.random.normal(0, 4.5)
+        Particle ids 2, 3, 5, 6 are electrons and muons,
+        id 4 is no longer used (were neutrino's).
+
+        The detector is approximated by a square with a surface of 0.5
+        square meter which is *not* correctly rotated.  In fact, during
+        the simulation, the rotation of the detector is undefined.  This
+        is faster than a more thorough implementation.
+
+        *Detector height is ignored!*
+
+        :param detector: detector for which to get particles.
+
+        """
+        x, y = detector.get_xy_coordinates()
+        detector_boundary = 0.3535534
+        query = ('(x >= %f) & (x <= %f) & (y >= %f) & (y <= %f)'
+                 ' & (particle_id >= 2) & (particle_id <= 6)' %
+                 (x - detector_boundary, x + detector_boundary,
+                  y - detector_boundary, x + detector_boundary))
+        return self.groundparticles.read_where(query)
 
 
 class DetectorBoundarySimulation(GroundParticlesSimulation):
@@ -227,7 +211,7 @@ class DetectorBoundarySimulation(GroundParticlesSimulation):
 
     """
 
-    def simulate_detector_response(self, detector, shower_parameters):
+    def get_particles_in_detector(self, detector):
         """Simulate the detector detection area accurately.
 
         First particles are filtered to see which fall inside a
@@ -252,17 +236,7 @@ class DetectorBoundarySimulation(GroundParticlesSimulation):
                   y - detector_boundary, x + detector_boundary,
                   line1, line1, line2, line2))
 
-        detected = [row['t'] for row in self.groundparticles.where(query)]
-        if detected:
-            n_detected = len(detected)
-            transporttimes = self.simulate_signal_transport_time(n_detected)
-            for i in range(n_detected):
-                detected[i] += transporttimes[i]
-            observables = {'n': n_detected, 't': min(detected)}
-        else:
-            observables = {'n': 0., 't': -999}
-
-        return observables
+        return self.groundparticles.read_where(query)
 
     def get_line_boundary_eqs(self, p0, p1, p2):
         """Get line equations using three points
@@ -310,69 +284,24 @@ class DetectorBoundarySimulation(GroundParticlesSimulation):
         return b1, line, b2
 
 
-class DetectorSignalSimulation(GroundParticlesSimulation):
+class ParticleCounterSimulation(GroundParticlesSimulation):
 
-    """ More accuratly simulate the detection area of the detectors.
+    """Do not simulate mips, just count the number of particles."""
 
-    This requires a slightly more complex query which is a bit slower.
+    def simulate_detector_mips(self, particle_momenta):
+        """A mip for a mip, count number of particles in a detector."""
+
+        return len(particle_momenta)
+
+
+class GroundParticlesSimulationWithoutErrors(ErrorlessSimulation,
+                                             GroundParticlesSimulation):
+
+    """This simulation does not simulate errors/uncertainties
+
+    This results in perfect timing (first particle through detector)
+    and particle counting for the detectors.
 
     """
 
-    def simulate_detector_response(self, detector, shower_parameters):
-        """Simulate the detector detection area accurately.
-
-        First particles are filtered to see which fall inside a
-        non-rotated square box around the detector (i.e. sides of 1.2m).
-        For the remaining particles a more accurate query is used to see
-        which actually hit the detector. The advantage of using the
-        square is that column indexes can be used, which may speed up
-        queries.
-
-        """
-        detector_boundary = 0.3535534
-        x, y = detector.get_xy_coordinates()
-
-        # particle ids 2, 3, 5, 6 are electrons and muons, and id 4 is no
-        # longer used (were neutrino's).
-        query = ('(x >= %f) & (x <= %f) & (y >= %f) & (y <= %f)'
-                 ' & (particle_id >= 2) & (particle_id <= 6)' %
-                 (x - detector_boundary, x + detector_boundary,
-                  y - detector_boundary, x + detector_boundary))
-        detected = [[row['t'], row['p_x'], row['p_y'], row['p_z']]
-                    for row in self.groundparticles.where(query)]
-
-        if detected:
-            n_detected = len(detected)
-            mips = 0.
-            for i in range(n_detected):
-                # determination of lepton angle of incidence
-                px = detected[i][1]
-                py = detected[i][2]
-                pz = detected[i][3]
-                costheta = abs(pz) / np.sqrt(px * px + py * py + pz * pz)
-
-                # Simulation of convoluted distribution of electron and
-                # muon energy losses with the scintillator response
-
-                y = random.random()
-
-                if y < 0.3394:
-                    mip = (0.48 + 0.8583 * np.sqrt(y)) / costheta
-                elif y < 0.4344:
-                    mip = (0.73 + 0.7366 * y) / costheta
-                elif y < 0.9041:
-                    mip = (1.7752 - 1.0336 * np.sqrt(0.9267 - y)) / costheta
-                else:
-                    mip = (2.28 - 2.1316 * np.sqrt(1 - y)) / costheta
-                mips += mip
-
-            # simulation of transport times
-
-            transporttimes = self.simulate_signal_transport_time(n_detected)
-            for i in range(n_detected):
-                detected[i][0] += transporttimes[i]
-            observables = {'n': mips, 't': np.min(detected, 0)[0]}
-        else:
-            observables = {'n': 0., 't': -999}
-
-        return observables
+    pass
